<?php
/**
 * Here is your custom functions.
 */

function wz_vue_parse($file) {
    $vueContent = file_get_contents($file);
    $parsed = [
        'template' => '',
        'script' => 'export default {}',
        'style' => ''
    ];
    if (preg_match('/<\s*template[^>]*>([\s\S]*)<\/\s*template\s*>/', $vueContent, $matches)) {
        $parsed['template'] = trim($matches[1]);
    }
    if (preg_match('/<\s*style[^>]*>([\s\S]*)<\/\s*style\s*>/', $vueContent, $matches)) {
        $parsed['style'] = trim($matches[1]);
    }
    if (preg_match('/<\s*script[^>]*>([\s\S]*)<\/\s*script\s*>/', $vueContent, $matches)) {
        if (strpos(trim($matches[1]), 'export default') !== false) {
            $parsed['script'] = trim($matches[1]);
        } else {
            $parsed['script'] = 'export default ' . trim($matches[1]);
        }
    }
    // 生成scoped ID
    $scopedId = 'scoped-' . bin2hex(random_bytes(4));
    $scopedStyles = '';
    
    if (!empty($parsed['style'])) {
        $scopedId = 'scoped-' . bin2hex(random_bytes(4));
        
        // 处理模板：确保根元素添加 data-scoped-id
        $scopedTemplate = preg_replace(
            '/^(\s*<[\w-]+)(\s|>)/m',
            "$1 data-{$scopedId}$2",
            $parsed['template'],
            1
        );
        
        // 处理 CSS 作用域
        $scopedStyles = preg_replace_callback(
            '/(@[^{]+\{[^}]+\})|([^{]+)\{([^}]*)\}/s', 
            function($matches) use ($scopedId) {
                if ($matches[1]) {
                    return $matches[1]; // 跳过 @规则
                }
                $selectors = array_map('trim', explode(',', $matches[2]));
                $scoped = array_map(function($sel) use ($scopedId) {
                    if (strpos($sel, ':global') !== false) {
                        return $sel;
                    }
                    return "[data-{$scopedId}]$sel"; 
                }, $selectors);
                    return implode(',', $scoped) . '{' . $matches[3] . '}';
            },
            $parsed['style']
            );
        
        
    }
    
    // 分离import语句和组件定义
    $importStatements = '';
    $componentDefinition = '{}';
    if (preg_match('/^export\s+default\s*(\{.*)$/s', $parsed['script'], $scriptMatches)) {
        $componentDefinition = $scriptMatches[1];
    }else if (preg_match('/^(.*?)(export\s+default\s*\{.*)$/s', $parsed['script'], $scriptMatches)) {
        $importStatements = trim($scriptMatches[1]);
        $componentDefinition = preg_replace('/^export\s+default/', '', $scriptMatches[2]);
    }
    
    // 生成最终JS代码
    $jsCode = <<<JS
{$importStatements}
const exportDefault = {$componentDefinition};
exportDefault.template = `{$scopedTemplate}`;
export default exportDefault;
JS;

// 只有在有样式时才添加样式相关的代码
if (!empty(trim($scopedStyles))) {
    $jsCode = <<<JS
const style = document.createElement('style');
style.textContent = `{$scopedStyles}`;
document.head.appendChild(style);
{$jsCode}
JS;
}
return $jsCode;
}

